﻿using SnapshotWinFormsApp.Entities.NewHotline;
using SnapshotWinFormsApp.Repository.Interfaces;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SnapshotWinFormsApp.Repository;

public class BaseRepository<T> : IBaseRepository<T> where T : Entity, new()
{
    private readonly SqlSugarClient _db;

    public BaseRepository(DbSqlServer context, string key)
    {
        _db = context.DbItems.GetValueOrDefault(key + "PGSQLDB");
    }

    public async Task<List<T>> GetAllAsync(CancellationToken token)
    {
        return _db.Queryable<T>().ToList();
    }

    public T GetById(int id)
    {
        return _db.Queryable<T>().InSingle(id);
    }

    public async Task<string> InsertAsync(T entity, CancellationToken token)
    {
        entity.InitId();
        await _db.Insertable(entity).ExecuteCommandAsync(token);
        return entity.Id;
    }

    public ISugarQueryable<T> Queryable()
    {
        return _db.Queryable<T>();
    }

    public void Update(T entity)
    {
        _db.Updateable<T>(entity).ExecuteCommand();
    }
}
